@@ -1,6 +1,6 @@
 ;;; mu4e-draft.el --- Helpers for m4e-compose -*- lexical-binding: t -*-
 
-;; Copyright (C) 2024 Dirk-Jan C. Binnema
+;; Copyright (C) 2024-2025 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
@@ -25,7 +25,6 @@
 ;; Implements various helper functions for mu4e-compose. This all
 ;; look a little convoluted since we need to subvert the gnus/message
 ;; functions a bit to work with mu4e.
-
 (require 'message)
 (require 'mu4e-config)
 (require 'mu4e-helpers)
@@ -363,7 +362,9 @@ appropriate sent-messages folder. If MSGPATH is nil, do nothing."
       (message-generate-headers '(Message-ID)))
     ;; older Emacsen (<= 28 perhaps?) won't update the Date
     ;; if there already is one; so make sure it's gone.
-    (message-remove-header "Date")
+    (save-restriction
+      (message-narrow-to-headers)
+      (message-remove-header "Date"))
     (message-generate-headers '(Date Subject From))
     (mu4e--delimit-headers 'undelimit))) ;; remove separator
 
@@ -387,26 +388,34 @@ also marked as Seen.
 
 Function assumes that it is executed in the context of the
 message buffer."
-  (when-let ((buf (find-file-noselect path)))
+  ;; note that we can't use mu4e-compose-parent-message here, since it
+  ;; no longer available when editing a draft. So we scan the outgoing
+  ;; message for the information.
+  (when-let* ((buf (find-file-noselect path)))
     (with-current-buffer buf
-      (let ((in-reply-to (message-field-value "in-reply-to"))
-            (forwarded-from)
-            (references (message-field-value "references")))
-        (unless in-reply-to
-          (when references
-            (with-temp-buffer ;; inspired by `message-shorten-references'.
-              (insert references)
-              (goto-char (point-min))
-              (let ((refs))
-                (while (re-search-forward "<[^ <]+@[^ <]+>" nil t)
-                  (push (match-string 0) refs))
-                ;; the last will be the first
-                (setq forwarded-from (car refs))))))
-        ;; remove the <> and update the flags on the server-side.
-        (when (and in-reply-to (string-match "<\\(.*\\)>" in-reply-to))
-          (mu4e--server-move (match-string 1 in-reply-to) nil "+R-N"))
-        (when (and forwarded-from (string-match "<\\(.*\\)>" forwarded-from))
-          (mu4e--server-move (match-string 1 forwarded-from) nil "+P-N"))))))
+      (let* ((in-reply-to (message-field-value "in-reply-to"))
+             (references (message-field-value "references"))
+             (forwarded-from
+              (unless (or in-reply-to (not references))
+                (with-temp-buffer ;; inspired by `message-shorten-references'.
+                  (insert references)
+                  (goto-char (point-min))
+                  (let ((refs))
+                    (while (re-search-forward "<[^ <]+@[^ <]+>" nil t)
+                      (push (match-string 0) refs))
+                    (car refs))))) ;; the last shall be the first
+             ;; remove the <>
+             (in-reply-to (and in-reply-to (string-match "<\\(.*\\)>" in-reply-to)
+                               (match-string 1 in-reply-to)))
+             (forwarded-from (and forwarded-from (string-match "<\\(.*\\)>" forwarded-from)
+                                  (match-string 1 forwarded-from))))
+        ;; mark parents.
+        (when in-reply-to
+          (mu4e-log 'misc "mark %s as Replied" in-reply-to)
+          (mu4e--server-move in-reply-to nil "+R-N"))
+        (when forwarded-from
+          (mu4e-log 'misc "mark %s as Passed (forwarded)" forwarded-from)
+          (mu4e--server-move forwarded-from nil "+P-N"))))))
 
 (defun mu4e--compose-after-save()
   "Function called immediately after the draft buffer is saved."
@@ -445,14 +454,14 @@ appropriate flag at the message forwarded or replied-to."
   ;; typically, draft is gone and the sent message appears in sent. Update flags
   ;; for related messages, i.e. for Forwarded ('Passed') and Replied messages,
   ;; try to set the appropriate flag at the message forwarded or replied-to.
-  (when-let ((fcc-path (message-field-value "Fcc")))
+  (when-let* ((fcc-path (message-field-value "Fcc")))
     (mu4e--set-parent-flags fcc-path)
     ;; we end up with a ((buried) buffer here, visiting the
     ;; fcc-path; not quite sure why. But let's get rid of it (#2681)
-    (when-let ((buf (find-buffer-visiting fcc-path)))
+    (when-let* ((buf (find-buffer-visiting fcc-path)))
       (kill-buffer buf)))
   ;; remove draft
-  (when-let ((draft (buffer-file-name)))
+  (when-let* ((draft (buffer-file-name)))
     (mu4e--server-remove draft)))
 
 (defun mu4e--compose-before-send ()
@@ -490,12 +499,18 @@ Creates a buffer NAME and returns it."
   (setq mu4e--message-buf (current-buffer)))
 
 (defun mu4e--message-is-yours-p ()
-  "Mu4e's override for `message-is-yours-p'."
-  (seq-some (lambda (field)
-              (if-let ((recip (message-field-value field)))
-                  (mu4e-personal-or-alternative-address-p
-                   (car (mail-header-parse-address recip)))))
-            '("From" "Sender")))
+  "Did you send the message we're responding to?
+Mu4e's override for `message-is-yours-p'. This only considers the
+From: address, not the Sender: address.
+
+Note that we check the parent, message-at-point so this should
+work both in headers-view and message-view."
+  (seq-some (lambda (addr)
+              (mu4e-personal-or-alternative-address-p addr))
+            (seq-map (lambda (addr) (plist-get addr :email))
+                     (plist-get (or mu4e-compose-parent-message
+                                    (mu4e-message-at-point 'noerror))
+                                :from))))
 
 (defmacro mu4e--validate-hidden-buffer (&rest body)
   "Macro to evaluate BODY and asserts that it yields a valid buffer.
@@ -596,7 +611,7 @@ COMPOSE-TYPE and PARENT are as in `mu4e--draft'."
     (set-visited-file-name ;; make it a draft file
      (mu4e--draft-message-path (mu4e--draft-basename) parent)))
   ;; fcc
-  (when-let ((fcc-path (mu4e--fcc-path (mu4e--draft-basename) parent)))
+  (when-let* ((fcc-path (mu4e--fcc-path (mu4e--draft-basename) parent)))
     (message-add-header (concat "Fcc: " fcc-path "\n")))
 
   (mu4e--prepare-draft-headers compose-type)
@@ -721,32 +736,32 @@ Returns the new buffer."
   ;; run pre-hook early, so user can influence later steps.
   (let ((mu4e-compose-parent-message parent)
         (mu4e-compose-type compose-type))
-    (run-hooks 'mu4e-compose-pre-hook))
+    (run-hooks 'mu4e-compose-pre-hook)
 
-  (mu4e--prepare-draft parent)
-  ;; evaluate BODY; this must yield a hidden, live buffer. This is evaluated in
-  ;; a temp buffer with contains the parent-message, if any. if there's a
-  ;; PARENT, load the corresponding message into a temp-buffer before calling
-  ;; compose-func
-  (let ((draft-buffer)
-        (oldframe (selected-frame))
-        (oldwinconf (current-window-configuration)))
-    (with-temp-buffer
-      ;; provide a temp buffer so the compose-func can do its thing
-      (setq draft-buffer (mu4e--validate-hidden-buffer (funcall compose-func)))
-      (with-current-buffer draft-buffer
-        ;; we have our basic buffer; turn it into a full mu4e composition
-        ;; buffer.
-        (mu4e--prepare-draft-buffer compose-type parent)))
-    ;; we're ready for composition; let's display it in the way user configured
-    ;; things: directly through display buffer (via pop-t or otherwise through
-    ;; mu4e-window.
-    (if (eq mu4e-compose-switch 'display-buffer)
-        (pop-to-buffer draft-buffer)
-      (mu4e-display-buffer draft-buffer 'do-select))
-    ;; prepare possible message actions (such as cleaning-up)
-    (mu4e--prepare-post oldframe oldwinconf)
-    draft-buffer))
+    (mu4e--prepare-draft parent)
+    ;; evaluate BODY; this must yield a hidden, live buffer. This is evaluated in
+    ;; a temp buffer with contains the parent-message, if any. if there's a
+    ;; PARENT, load the corresponding message into a temp-buffer before calling
+    ;; compose-func
+    (let ((draft-buffer)
+          (oldframe (selected-frame))
+          (oldwinconf (current-window-configuration)))
+      (with-temp-buffer
+        ;; provide a temp buffer so the compose-func can do its thing
+        (setq draft-buffer (mu4e--validate-hidden-buffer (funcall compose-func)))
+        (with-current-buffer draft-buffer
+          ;; we have our basic buffer; turn it into a full mu4e composition
+          ;; buffer.
+          (mu4e--prepare-draft-buffer compose-type parent)))
+      ;; we're ready for composition; let's display it in the way user
+      ;; configured things: directly through display buffer (via pop-t or
+      ;; otherwise through mu4e-window.
+      (if (eq mu4e-compose-switch 'display-buffer)
+          (pop-to-buffer draft-buffer)
+        (mu4e-display-buffer draft-buffer 'do-select))
+      ;; prepare possible message actions (such as cleaning-up)
+      (mu4e--prepare-post oldframe oldwinconf)
+      draft-buffer)))
 
 (defun mu4e--draft-with-parent (compose-type parent compose-func)
   "Draft a message based on some parent message.
